rm(list=ls())
setwd(" your directory")
data.df = read.table("object_detection_data.csv", header = T, sep = ";", dec = ".")
str(data.df)

#--------- research aim / Q -----------------------------------------------------------------------
# Dependency of the probability of the fluorescence trait detection success ("identified") of the 
# distance between the UV radiation source spot projection center ("dtl") on the ground and 
# the fluorescence trait [cm] for two different distance levels between the UAS sensor and the trait 
# (**dts** class 1: 2,2 m to 2,7 m, class 2: 3 m to 3,65 m) 
# and two levels of trait size (**size** class 1: 1 mm, class 2: 4 mm).

#---------- packages -----------------------------------------------------------------------------
library(MASS)
library(car)
library(fitdistrplus)
library(effects)
library(lme4)
library(MuMIn)

#----------- data inspection and preparation ------------------------------------------------------
#--- Metadata: ---#
# id = trait id / unique identifier ranging 1-1308
# size= trait size in mm, i.e. 1 and 4
# identified: 1=yes , 0=no
# image= the image data the data refers to with pre-defined
# dts = distance-to-sensor: distance between UAS sensor and the trait [m], class 1 = ranging 2.2-2.7m, class 2 = ranging 3-3.65m
# dtl= distance-to-light: distance between the UV light radiation source spot projection center on the ground and the fluorescence trait [cm]

# convert data.df$size and data.df$dts to a factor
data.df$size = as.factor(as.character(data.df$size))
data.df$dts = as.factor(as.character(data.df$dts))

# calculate dtl in cm: 
# converting distance measures from the QGIS-project into SI distance [cm], using the length of a DIN A2-page
data.df$dtl = 21*2/10183.749*10 *data.df$dtl

# data structure
table(data.df$identified)
table(data.df$dts) 
table(data.df$size) 
table(data.df$image) 
table(data.df$image, data.df$size)
table(data.df$image, data.df$identified)

tapply(data.df$identified, data.df$size, function(x)length(unique(x)) ) 
tapply(data.df$identified, data.df$dts, function(x)length(unique(x)) ) 

## what is the level of replication? missing values?
sapply(data.df, function(x) length(unique(x)))
sapply(data.df,function(x) sum(is.na(x)))

#visual inspections
plot(data.df$identified,data.df$dtl) 
plot(data.df$size,data.df$dtl)
plot(data.df$dts,data.df$dtl)
plot(data.df$image,data.df$dtl) 
plot(data.df$dts,data.df$image) 
plot(data.df$dts,data.df$size)

#--- conclusion from inspection: ---#
# we have a dependend, unbalanced data structure with a different number of images from different heights.
# Moreover, we have several measures (id) per image => level of pseudoreplication
# solution: we need mixed-effects models to cope with this
# the response variable consists of 2 levels, yes=1, no=0 - i.e. we have a binary response variable 
# thus we need to apply logistic regression, glm(m) with family=binomial
# note: no overdispersion with a binary response variable

#-------------- analysis -----------------------------------------------------------------------------

# checking disribution of the continous predictor (variable "dtl")
plot(with(data.df,fitdist(dtl,"norm")))
plot(with(data.df,fitdist(dtl,"lnorm")))
plot(with(data.df,fitdist(dtl,"gamma")))

with(data.df,fitdist(dtl,"norm")$aic) # => predictor is normally distributed
with(data.df,fitdist(dtl,"lnorm")$aic) #
with(data.df,fitdist(dtl,"gamma")$aic) #

#---------------------------------------------------#
#--- entering the world of mixed effects models ---#

# random intercept model with interactions amongst predictors in fixed effects, 
# accounting for pseudoreplication in random effects part
mod.full <- glmer(identified ~ size * dts * dtl +(1|dts/image), data=data.df,family=binomial
              , glmerControl(optimizer="bobyqa", optCtrl = list(maxfun = 200000)) # increased iterations-steps to prevent non-convergence
              )
# conduct model selection based on AICc
options(na.action=na.fail)
dredge(mod.full, trace=T)

# convergence problems at models with more than one interaction ongoing => the scale range of the predictor variables is different
# we divide our continous predictor dtl by 10 to reduce the span
mod.1 <- glmer(identified ~ size * dts * I(dtl/10) +(1|dts/image), data=data.df,family=binomial
                  , glmerControl(optimizer="bobyqa", optCtrl = list(maxfun = 200000)) # increased iterations-steps to prevent non-convergence
)
#
options(na.action=na.fail)
dredge(mod.1, trace=T)

# top model with the highest weight:
top1 <- glmer(identified ~ size + dts + I(dtl/10) + I(dtl/10):size +(1|dts/image), data=data.df,family=binomial)

#--- graphical model inspection ---#
plot(top1)

plot(Effect(c("dtl","size","dts"),top1, residuals=T),
     partial.residual=list(lty="dashed") )
# the patterns for the graphs from the plot command look awkward at first glance
# since we have a Bernoulli distributed (binary) glm, we see everything "doubled"
# see discussion in "Zuur-Mixed Effects Models and Extensions in Ecology with R", p.253.
# we keep our model choice

#-------------- results ------------------------------------

# redo top1 without the dts range reduction
top.new <- glmer(identified ~ size + dts + dtl + dtl:size +(1|dts/image), data=data.df,family=binomial)
Anova(top.new, type=2) 
summary(top.new)
ranef(top.new)

#--------------------- graphics-----------------------------
# using library(effects)
plot(allEffects(top.new))
plot(predictorEffects(top.new))

# single effects
m1 <- Effect("dts",top.new)
m2 <- Effect(c("dtl","size"),top.new)

plot(m1,main="",xlab="dts",ylab="Probability of trace detection"
     ,lattice=list(array=list(row=1,col=1,nrow=1,ncol=2,more=T)))
plot(m2,main="",xlab="dtl",ylab="Probability of trace detection"
     ,lattice=list(array=list(row=1,col=2,nrow=1,ncol=2,more=F)))

# all effects plotted
m3 <- Effect(c("dtl","size","dts"),top.new)
m3$variables$dts$levels <- c("DTS 1", "DTS 2")
levels(m3$x$dts) <- c("DTS 1", "DTS 2")

#pdf("Figure 3.pdf",width=6,height=6)
svg("Figure 3.svg",width=6,height=6)
plot(m3,main="",xlab="DTL [cm]",ylab="Probability of trace detection"
     ,alternating=FALSE
     ,factor.names=FALSE
     ,lattice=list(
       key.args= list(x=.05,y=.3, columns=1, border=T ) )
     ,multiline=T ,ci.style="bands",rescale.axis=FALSE
    )
dev.off()

## => refine legend labelling in Inkscape

##
## --------------------------------- end
##
##
##